# read in family info: parents/probands/other relavtives; genders; affected or not
# read in SV

#! /h/jingqichen/localperl/bin/perl
use warnings;
use File::Basename qw(dirname);
use Cwd qw(abs_path);
use lib dirname(abs_path $0);

require SV_SUB;

$sv_file = $ARGV[0]; # $hg=$ARGV[1];
$tad_bd_file = "data/tad/boundary_regions_hg38.bed"; #sorted
my $gene_anno;
opendir my $dir, "data/gene_anno" or die;
my @annos = readdir $dir;
for $an(@annos){
$gene_anno = "data/gene_anno/".$an if $an =~ "Homo_sapiens.GRCh38.*.PC.genes.bed\$"; # sorted
}
$re_anno = "data/reg_anno/combined_regulatory_elements_for_SVint_hg38.bed"; #sorted
$rg_all = "data/target/consolidated_RE2target_PCgenes.txt";
$kg_fi_file = "data/kg/ALL.wgs.mergedSV.v8.20130502.svs.genotypes.hg38.vcf_based_FI.txt";
print "SVint v1.2 ...\n";
print "warning: SVint is using $kg_fi_file\n";

print "Read in data...\n";
($same_a, $diff_a, $lines_a) = SV_SUB::read_sv($sv_file);
# same chr, diff chr; in this tool only SVs that don't cross different chr are used for analysis
%same_a = %{$same_a}; %diff_a = %{$diff_a}; %lines_a = %{$lines_a};
# $same_a{$var}{$type}=$gt; $diff_a{"$loc1\t$loc2"}{$type}=$gt; $lines_a{$id}{$line}=$var or "$loc1\t$loc2";

# filter: only use intergenic SVs
print "Filter by gene annotation...\n";
&SV_SUB::write_bed($same_a, "$sv_file\_temp"); # output $sv_file\_temp.sorted.bed
$overlap_gene = SV_SUB::overlap2("$sv_file\_temp.sorted.bed", $gene_anno);
%overlap_gene = %{$overlap_gene};
my %see; map{$see{$_}=1}(keys %overlap_gene);
my %to_use;
for $var(keys %same_a){
	for $tp(keys %{$same_a{$var}}){
		next if exists $see{$var}; $to_use{$var}{$tp}=$same_a{$var}{$tp};
	}
}
&SV_SUB::write_bed(\%to_use, "$sv_file\_temp2"); # output $sv_file\_temp2.sorted.bed

# annotate: with TAD boundary regions
print "Calculate SV impact on TAD boundary regions ...\n";
($sv2tad_bd) = SV_SUB::anno_tad1("$sv_file\_temp2.sorted.bed", $tad_bd_file);

# annotate: with regulatory elements
print "Annotate with regulatory elements...\n";
$sv2re = SV_SUB::overlap("$sv_file\_temp2.sorted.bed", $re_anno); #$sv2re{$var}{$re} = overlapped length ratio in RE; $var & $re: "$chr\t$stt\t$end"

# annotate with each regulatory element - target gene file:
print "Annotate with RE-target information, and their TAD boudary impact ...\n";
($sv2target, $loc_gn) = SV_SUB::rg_anno($sv2re, $re_anno, $rg_all);
&SV_SUB::write_bed3($loc_gn, $gene_anno, "$sv_file\_temp3"); # output $sv_file\_temp3.sorted.bed
($lg2tad_bd) = SV_SUB::anno_tad2("$sv_file\_RegulatoryRegion--Gene_crossTAD-cells.txt", "$sv_file\_temp3.sorted.bed", $tad_bd_file);
%lg2tad_bd = %{$lg2tad_bd};

# annotate with FIR
print "Annotate with Frequency of Impact for REs calculated through 1000genome SV data...\n";
$fi = SV_SUB::read_fi($kg_fi_file);
$sv2fi_tad = SV_SUB::anno_fi_tad($sv2target, \%to_use, $fi, $lg2tad_bd);

# write out new VCF
print "Write out the new VCF...\n";
&SV_SUB::write_svint_anno($sv_file."_SVint_v1.2_anno.vcf", $sv_file, $lines_a, $same_a, \%to_use, \%overlap_gene, $sv2tad_bd, $sv2fi_tad);
print "SVint done.\n";

system "rm $sv_file\_temp.*bed"; system "rm $sv_file\_temp2.*bed"; system "rm $sv_file\_temp3.*bed*";

